"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const cli_flags_1 = require("cli-flags");
const cli_engine_config_1 = require("cli-engine-config");
const http_call_1 = require("http-call");
const help_1 = require("./help");
const util_1 = require("util");
const pjson = require('../package.json');
const debug = require('debug')('cli-engine-command');
class Command {
    constructor(config) {
        this.hidden = false;
        this.aliases = [];
        this.strict = true;
        this.variableArgs = false;
        this.Flags = {};
        this.Args = [];
        this._version = pjson.version;
        this.config = cli_engine_config_1.buildConfig(config);
        const { CLI } = require('cli-ux');
        this.cli = new CLI({ debug: this.config.debug, mock: this.config.mock, errlog: this.config.errlog });
        this.http = http_call_1.HTTP.defaults({
            headers: {
                'user-agent': `${this.config.name}/${this.config.version} (${this.config.platform}-${this.config
                    .arch}) node-${process.version}`,
            },
        });
    }
    get id() {
        if (this.name)
            return this.name;
        let cmd = [];
        if (this.topic)
            cmd.push(this.topic);
        if (this.command)
            cmd.push(this.command);
        let id = cmd.join(':');
        if (id)
            return id;
        let ctor = this.constructor;
        return ctor.name;
    }
    static mock(...argv) {
        return __awaiter(this, void 0, void 0, function* () {
            argv.unshift('argv0', 'argv1');
            const cmd = (yield this.run({ argv, mock: true }));
            return {
                cmd,
                stdout: cmd.cli.stdout.output,
                stderr: cmd.cli.stderr.output,
            };
        });
    }
    static run(config) {
        return __awaiter(this, void 0, void 0, function* () {
            const cmd = new this(config);
            try {
                debug('initializing %s version: %s', cmd.id, cmd._version);
                debug('argv: %o', cmd.config.argv);
                yield cmd.init();
                debug('run');
                yield cmd.run();
                debug('done');
                yield cmd.cli.done();
            }
            catch (err) {
                cmd.cli.error(err);
            }
            return cmd;
        });
    }
    get out() {
        util_1.deprecate(() => { }, 'this.out is deprecated, use this.cli');
        return this.cli;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.cli.handleUnhandleds();
            if (this.variableArgs) {
                util_1.deprecate(() => { }, 'variableArgs is deprecated. Use strict = true instead.');
            }
            const { argv, flags, args } = yield cli_flags_1.parse({
                flags: this.Flags,
                args: this.Args,
                strict: this.strict !== false && !this.variableArgs,
                argv: this.config.argv.slice(2),
            });
            this.flags = flags;
            this.argv = argv;
            this.args = args;
        });
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    buildHelp() {
        let help = new help_1.Help(this.config);
        return help.command(this);
    }
    buildHelpLine() {
        let help = new help_1.Help(this.config);
        return help.commandLine(this);
    }
}
exports.Command = Command;
